library httpserver;

import "dart:io" as io;
import "stream_binder.dart";
import "dart:async";

class WebSocketBindHttpServer{
  
  final String _filepath;
  StreamController<ConnectionEvent> _binder_controller = new StreamController();
  Stream<ConnectionEvent> get onWebSocketConnection => _binder_controller.stream;
  WebSocketBindHttpServer(String host,int port,this._filepath){
    io.HttpServer.bind(host, port).then(_onHttpServer);
  }

  void _onHttpServer(io.HttpServer server) {
    server.listen(_onRequest);
  }
  void _onRequest(io.HttpRequest request) {
    if (io.WebSocketTransformer.isUpgradeRequest(request)) {
      io.WebSocketTransformer.upgrade(request).then((ws) => _onWebSocket(ws,request.cookies));
    } else {
      _onHttp(request);
    }
  }
  
  void _onWebSocket(io.WebSocket ws,List<io.Cookie> cookies){
    var binder = new Binder(ws,ws.add);
    var connection_event = new ConnectionEvent()..binder = binder..cookies = cookies;
    _binder_controller.add(connection_event);
  }
  
  void _onHttp(io.HttpRequest req) {
    
    if(req.uri.path.contains("..")){
      req.response
        ..statusCode = io.HttpStatus.BAD_REQUEST
        ..close()
        ;
    }
  
    var head = _getHead(req.uri.path);
    var path = _filepath + req.uri.path;
    if (req.uri.path == "/") {
      path += "index.html";
    }
    try {
      req.response.headers.add("content-type", head);
      new io.File(path).openRead().pipe(req.response).then((_) {}, onError: (e) {
        req.response.close();
      });
    } catch (e) {
      req.response.close();
    }
  
  }
  
  String _getHead(String req) {
    try {
      if (req.endsWith("dart")) {
        return "application/dart";
      } else if (req.endsWith("css")) {
        return "text/css";
      } else if (req.endsWith("js")) {
        return "text/javascript";
      }
      return "text/html";
    } catch (e) {
      return "text/html";
    }
  }
}

class ConnectionEvent{
  Binder binder;
  List<io.Cookie> cookies;
}